---
id: working
title: Working
sidebar_label: Working
---

# Working

## Managing issues

Well crafted issues improve the speed that work can be triaged, understood, and completed. To help guide this process [issue templates](https://github.com/Microsoft/fast-dna/tree/master/.github/ISSUE_TEMPLATE) exist to allow users to choose an appropriate [bug report](https://github.com/Microsoft/fast-dna/blob/master/.github/ISSUE_TEMPLATE/report-a-bug.md) or [feature request](https://github.com/Microsoft/fast-dna/blob/master/.github/ISSUE_TEMPLATE/request-a-feature.md).

Here's an excellent resource for [do's and don'ts](https://hackernoon.com/45-github-issues-dos-and-donts-dfec9ab4b612) when working with bugs, issues, and requests.

[GitHub labels](https://developer.github.com/v3/issues/labels/) are used to manage issues broadly considered work items grouped by color and prefixed by label name.

## Pull requests

When landing a pull request (PR), individuals must modify the original commit message to include comprehensive details. This detail along with other git history meta information deploys as part of the documentation changelog history. Changelogs are generated by [Lerna](https://lernajs.io/) using [conventional commits](https://conventionalcommits.org/).

PR commit messages should include the issue number per the [github closing issue keywords](https://help.github.com/articles/closing-issues-using-keywords/) documentation.

One or more collaborators are automatically assigned to review each PR based on areas of [ownership](https://github.com/Microsoft/fast-dna/blob/master/.github/CODEOWNERS).

When a peer review is created, a [template](https://github.com/Microsoft/fast-dna/blob/master/.github/pull_request_template.md) is provided that includes the best practices and processes for a review message.

### Interim commits

The above outlines the format for the **final** commit, but contributors have many interim commits while working on their branch. These commits should be succinct, concise, and consistent for best readability. Always remember, a diff tells you _what_ changed in a commit, but only the commit message can adequately tell you _why_.

Typical interim commits may look like the following:

```bash
5ba3db6 update toggle markup for accessibility
84564a0 add border styles
887815f remove toggle underline on hover
```

Before submitting a pull request, [rebase](https://www.atlassian.com/git/tutorials/merging-vs-rebasing) your branch from master or use the *update* button provided by Github.

## Common commands

All "packages" share common script names for consistency along with their documentation guides.

To clean all packages node_modules:

```bash
lerna clean
```

To clean dependencies hoisted by yarn, navigate to the root of the project and run:

```bash
rm -rf node_modules
```

Install remote dependencies, build, and symlink local dependencies:

```bash
yarn install
lerna run prepare
```

To run all tests on all packages:

```bash
lerna run test
```

To run all tests on a single package:

```bash
cd packages/package-name
yarn test
```

Most packages have a _watch_ command that rebuilds the package's distribution when a file changes. This process can be useful when doing development across multiple packages:

```bash
yarn watch
```

## Troubleshooting

If you run into any errors and they seem unrelated to your work try the following procedure:

Delete root node_modules (`fast-dna/node_modules`) then:

```bash
lerna clean
yarn install
lerna run test
```

See if your issue reproduces.